@@ -153,16 +153,11 @@ sub run {
     my ($self, $request) = @_;
 
     my $content  = $self->execute();
-    my $response = Dancer::Response->current;
+    my $response = Dancer::SharedData->response;
 
-    if ( $response->is_forwarded ) {
-        my $new_req = Dancer::Request->new_for_request(
-            $request->method,
-            $response->{forward},
-            $request->params,
-            $request->body,
-            $request->headers,
-        );
+    if ( $response && $response->is_forwarded ) {
+        my $new_req = 
+            Dancer::Request->forward($request, $response->{forward});
 
         my $marshalled = Dancer::Handler->handle_request($new_req);
 
@@ -173,7 +168,8 @@ sub run {
         );
     }
 
-    if ($response->has_passed) {
+    if ($response && $response->has_passed) {
+        $response->pass(0);
         if ($self->next) {
             my $next_route = $self->find_next_matching_route($request);
             return $next_route->run($request);
@@ -187,14 +183,15 @@ sub run {
     # prevent warnings
     $content = (defined $content) ? $content : '';
 
-    # drop content if HEAD request
-    $content = '' if $request->is_head;
+    my $ct =
+      ( defined $response && defined $response->content_type )
+      ? $response->content_type()
+      : setting('content_type');
+
+    my $st = defined $response ? $response->status : 200;
 
-    # init response headers
-    my $ct = $response->content_type() || setting('content_type');
-    my $st = $response->status()       || 200;
     my $headers = [];
-    push @$headers, @{$response->headers_to_array};
+    push @$headers, @{ $response->headers_to_array } if defined $response;
 
     # content type may have already be set earlier
     # (eg: with send_error)
